@@ -1,18 +1,18 @@
-subroutine ks(ydimt, timevar, zt, tt, ht, rtv, qt, at, pt, vtuni, ftuni, ktuni, ahat, vvt, &
-     rt, rt0, rt1, nt, nt0, nt1, nt2, epshat, epshatvar, etahat, etahatvar, pinf, pstar, kinfuni,&
-     kstaruni, finfuni, fstaruni, d, j, p, m, r, n, eps)
+subroutine ks(ydimt, timevar, zt, tt, ht, at, pt, vtuni, ftuni, ktuni, ahat, vvt, &
+     rt, rt0, rt1, nt, nt0, nt1, nt2, pinf, pstar, kinfuni,&
+     kstaruni, finfuni, fstaruni, d, j, p, m, n, eps)
 
 implicit none
 
-integer, intent(in) :: d, j, p, m, r, n
+integer, intent(in) :: d, j, p, m, n
 integer :: t, i, k, l
 integer, intent(in), dimension(n) :: ydimt
-integer, intent(in), dimension(4) :: timevar
+integer, intent(in), dimension(3) :: timevar
 double precision, intent(inout), dimension(p,m,n) :: zt  
 double precision, intent(in), dimension(m,m,(n-1)*timevar(1)+1) :: tt 
 double precision, intent(inout), dimension(p,p,n) :: ht
-double precision, intent(in), dimension(m,r,(n-1)*timevar(2)+1) :: rtv
-double precision, intent(in), dimension(r,r,(n-1)*timevar(3)+1) :: qt
+!double precision, intent(in), dimension(m,r,(n-1)*timevar(2)+1) :: rtv
+!double precision, intent(in), dimension(r,r,(n-1)*timevar(3)+1) :: qt
 double precision, intent(in), dimension(m,n+1) :: at
 double precision, intent(in), dimension(m,m,n+1) :: pt
 double precision, intent(in), dimension(p,n) ::  vtuni
@@ -24,8 +24,8 @@ double precision, intent(inout), dimension(m,n) :: ahat
 double precision, intent(inout), dimension(m,m,n) :: vvt
 double precision, intent(inout), dimension(m,m,d+1) ::  pinf
 double precision, intent(in), dimension(m,m,d+1) ::  pstar
-double precision, intent(in),dimension(m,p,n) ::  kinfuni
-double precision, intent(in),dimension(m,p,n) ::  kstaruni
+double precision, intent(in),dimension(m,p,d) ::  kinfuni
+double precision, intent(in),dimension(m,p,d) ::  kstaruni
 double precision, intent(in),dimension(p,d) ::  fstaruni
 double precision, intent(in), dimension(p,d) ::  finfuni
 double precision, dimension(m,m) :: linfuni
@@ -36,10 +36,6 @@ double precision, intent(inout), dimension(m,d+1) :: rt1
 double precision, intent(inout), dimension(m,m,d+1) :: nt0
 double precision, intent(inout), dimension(m,m,d+1) :: nt1
 double precision, intent(inout), dimension(m,m,d+1) :: nt2
-double precision, intent(inout), dimension(p,n) :: epshat
-double precision, intent(inout), dimension(p,n) :: epshatvar
-double precision, intent(inout), dimension(r,n) :: etahat
-double precision, intent(inout), dimension(r,r,n) :: etahatvar
 double precision, dimension(m,m) ::  lt
 double precision, dimension(m,m) :: nrec
 double precision, dimension(m,m) :: nrec1
@@ -47,9 +43,9 @@ double precision, dimension(m,m) :: nrec2
 double precision, dimension(m) :: rrec
 double precision, dimension(m) :: rrec1
 double precision, dimension(m) :: rhelp
-double precision, dimension(r) :: r1
-double precision, dimension(m,r) :: mr
-double precision, dimension(m,r) :: mr2
+!double precision, dimension(r) :: r1
+!double precision, dimension(m,r) :: mr
+!double precision, dimension(m,r) :: mr2
 double precision, dimension(m,m) ::  im
 double precision, dimension(m,m) ::  mm
 double precision, dimension(m,m) ::  mm2
@@ -148,6 +144,9 @@ rt(1:m,n+1) = 0.0d0
 do t = n, d+1, -1 !do until diffuse starts
    do i = ydimt(t), 1 , -1       
       if(abs(ftuni(i,t)) > eps) then 
+       !  epshat(i,t) = ht(i,i,t)*(vtuni(i,t)/ftuni(i,t) - ddot(m,ktuni(1:m,i,t),1,rrec,1))
+       !  call dgemv('n',m,m,1.0d0,nrec,m,ktuni(1:m,i,t),1,0.0d0,rhelp,1) !beta oli 1.0d0
+       !  epshatvar(i,t) = ht(i,i,t) - (ht(i,i,t)**2)*(1.0d0/ftuni(i,t) + ddot(m,ktuni(1:m,i,t),1,rhelp,1))  
          lt = im
          call dger(m,m,-1.0d0,ktuni(1:m,i,t),1,zt(i,1:m,t),1,lt,m) !l = I -kz 
          call dgemv('t',m,m,1.0d0,lt,m,rrec,1,0.0d0,rhelp,1) 
@@ -156,19 +155,16 @@ do t = n, d+1, -1 !do until diffuse starts
          call dsymm('l','u',m,m,1.0d0,nrec,m,lt,m,0.0d0,mm,m) !n*l
          call dgemm('t','n',m,m,m,1.0d0,lt,m,mm,m,0.0d0,nrec,m) !n = l'nl
          call dger(m,m,(1.0d0/ftuni(i,t)),zt(i,1:m,t),1,zt(i,1:m,t),1,nrec,m) ! n = n+z'z/f
-         !call dsyr('u',m,(1.0d0/ftuni(i,t)),zt(i,1:m,(t-1)*timevar(1)+1),1,nrec,m) ! n = n+z'z/f      
-         epshat(i,t) = ht(i,i,t)*(vtuni(i,t)/ftuni(i,t) - ddot(m,ktuni(1:m,i,t),1,rrec,1))
-         call dgemv('n',m,m,1.0d0,nrec,m,ktuni(1:m,i,t),1,1.0d0,rhelp,1)
-         epshatvar(i,t) = ht(i,i,t) - (ht(i,i,t)**2)*(1.0d0/ftuni(i,t) + ddot(m,ktuni(1:m,i,t),1,rhelp,1))  
+         !call dsyr('u',m,(1.0d0/ftuni(i,t)),zt(i,1:m,(t-1)*timevar(1)+1),1,nrec,m) ! n = n+z'z/f         
       end if   
    end do
 
-   call dgemv('t',m,r,1.0d0,rtv(1:m,1:r,(t-1)*timevar(2)+1),m,rrec,1,0.0d0,r1,1)
-   call dsymv('u',r,1.0d0,qt(1:r,1:r,(t-1)*timevar(3)+1),r,r1,1,0.0d0,etahat(1:r,t),1)
-   etahatvar(1:r,1:r,t) = qt(1:r,1:r,(t-1)*timevar(3)+1)
-   call dsymm('r','u',m,r,1.0d0,qt(1:r,1:r,(t-1)*timevar(3)+1),r,rtv(1:m,1:r,(t-1)*timevar(2)+1),m,0.0d0,mr,m)
-   call dgemm('n','n',m,r,m,1.0d0,nrec,m,mr,m,0.0d0,mr2,m)
-   call dgemm('t','n',r,r,m,1.0d0,mr,m,mr2,m,1.0d0,etahatvar(1:r,1:r,t),r)   
+  ! call dgemv('t',m,r,1.0d0,rtv(1:m,1:r,(t-1)*timevar(2)+1),m,rrec,1,0.0d0,r1,1)
+  ! call dsymv('u',r,1.0d0,qt(1:r,1:r,(t-1)*timevar(3)+1),r,r1,1,0.0d0,etahat(1:r,t),1)
+  ! etahatvar(1:r,1:r,t) = qt(1:r,1:r,(t-1)*timevar(3)+1)
+  ! call dsymm('r','u',m,r,1.0d0,qt(1:r,1:r,(t-1)*timevar(3)+1),r,rtv(1:m,1:r,(t-1)*timevar(2)+1),m,0.0d0,mr,m)
+  ! call dgemm('n','n',m,r,m,1.0d0,nrec,m,mr,m,0.0d0,mr2,m)
+  ! call dgemm('t','n',r,r,m,1.0d0,mr,m,mr2,m,1.0d0,etahatvar(1:r,1:r,t),r)   
  
    call dcopy(m,rrec,1,rt(1:m,t),1) !r_t-1 = r_t,0
    nt(1:m,1:m,t) = nrec !n_t-1 = n_t,0
@@ -190,6 +186,9 @@ if(d>0) then
    if(ydimt(t)>j) then
       do i = ydimt(t), (j+1) , -1  
          if(ftuni(i,t) > eps) then 
+         !   epshat(i,t) = ht(i,i,t)*(vtuni(i,t)/ftuni(i,t) - ddot(m,ktuni(1:m,i,t),1,rrec,1))
+         !   call dgemv('n',m,m,1.0d0,nrec,m,ktuni(1:m,i,t),1,0.0d0,rhelp,1)
+         !   epshatvar(i,t) = ht(i,i,t) - (ht(i,i,t)**2)*(1.0d0/ftuni(i,t) + ddot(m,ktuni(1:m,i,t),1,rhelp,1))
             lt = im
             call dger(m,m,-1.0d0,ktuni(1:m,i,t),1,zt(i,1:m,t),1,lt,m) !l = i -kz
             call dgemv('t',m,m,1.0d0,lt,m,rrec,1,0.0d0,rhelp,1) 
@@ -199,9 +198,6 @@ if(d>0) then
             call dgemm('n','n',m,m,m,1.0d0,nrec,m,lt,m,0.0d0,mm,m) !n*l
             call dgemm('t','n',m,m,m,1.0d0,lt,m,mm,m,0.0d0,nrec,m) !n = l'nl
             call dger(m,m,(1.0d0)/ftuni(i,t),zt(i,1:m,t),1,zt(i,1:m,t),1,nrec,m) ! n = n+z'z/f      
-            epshat(i,t) = ht(i,i,t)*(vtuni(i,t)/ftuni(i,t) - ddot(m,ktuni(1:m,i,t),1,rrec,1))
-            call dgemv('n',m,m,1.0d0,nrec,m,ktuni(1:m,i,t),1,0.0d0,rhelp,1)
-            epshatvar(i,t) = ht(i,i,t) - (ht(i,i,t)**2)*(1.0d0/ftuni(i,t) + ddot(m,ktuni(1:m,i,t),1,rhelp,1))
          end if
       end do
       rrec1 = 0.0d0
@@ -216,6 +212,9 @@ if(d>0) then
 
    do i = j, 1, -1 
       if(finfuni(i,t)>eps) then
+       !  epshat(i,t) = -ht(i,i,t)/finfuni(i,t)*ddot(m,kinfuni(1:m,i,t),1,rrec,1)
+       !  call dgemv('n',m,m,1.0d0,nrec,m,kinfuni(1:m,i,t),1,0.0d0,rhelp,1)
+       !  epshatvar(i,t) = ht(i,i,t) - (ht(i,i,t)**2)/(finfuni(i,t)**2)*ddot(m,kinfuni(1:m,i,t),1,rhelp,1)
          linfuni = im            
          call dger(m,m,-1.0d0/finfuni(i,t),kinfuni(1:m,i,t),1,zt(i,1:m,t),1,linfuni,m) !linf
          rhelp = -kstaruni(1:m,i,t)
@@ -253,10 +252,12 @@ if(d>0) then
       
          call dgemm('t','n',m,m,m,1.0d0,linfuni,m,mm,m,0.0d0,nrec,m) !nt0 = linf'*mm
 
-         epshat(i,t) = -ht(i,i,t)/finfuni(i,t)*ddot(m,kinfuni(1:m,i,t),1,rrec,1)
-         call dgemv('n',m,m,1.0d0,nrec,m,kinfuni(1:m,i,t),1,0.0d0,rhelp,1)
-         epshatvar(i,t) = ht(i,i,t) - (ht(i,i,t)**2)/(finfuni(i,t)**2)*ddot(m,kinfuni(1:m,i,t),1,rhelp,1)
+        
       else
+        ! epshat(i,t) = ht(i,i,t) * (vtuni(i,t)/fstaruni(i,t) - ddot(m,kinfuni(1:m,i,t),1,rrec,1)/fstaruni(i,t))
+        ! call dgemv('n',m,m,1.0d0,nrec,m,kstaruni(1:m,i,t),1,0.0d0,rhelp,1)
+       !  epshatvar(i,t) = ht(i,i,t) - ht(i,i,t)**2 * (1.0d0/fstaruni(i,t) + &
+       !       ddot(m,kstaruni(1:m,i,t),1,rhelp,1)/(fstaruni(i,t)**2))
          lstaruni= im
          call dger(m,m,(-1.0d0)/fstaruni(i,t),kstaruni(1:m,i,t),1,zt(i,1:m,t),1,lstaruni,m) !lstar = I -Kstar*Z/Fstar         
          call dgemv('t',m,m,1.0d0,lstaruni,m,rrec,1,0.0d0,rhelp,1)
@@ -271,20 +272,16 @@ if(d>0) then
          call dgemm('n','n',m,m,m,1.0d0,nrec1,m,lstaruni,m,0.0d0,mm,m) !mm = nt1*lstar
          nrec1 = mm
          call dgemm('n','n',m,m,m,1.0d0,nrec2,m,lstaruni,m,0.0d0,mm,m) !mm = nt1*lstar
-         nrec2 = mm
-      
-         epshat(i,t) = ht(i,i,t) * (vtuni(i,t)/fstaruni(i,t) - ddot(m,kinfuni(1:m,i,t),1,rrec,1)/fstaruni(i,t))
-         call dgemv('n',m,m,1.0d0,nrec,m,kstaruni(1:m,i,t),1,0.0d0,rhelp,1)
-         epshatvar(i,t) = ht(i,i,t) - ht(i,i,t)**2 * (1.0d0/fstaruni(i,t) + &
-              ddot(m,kstaruni(1:m,i,t),1,rhelp,1)/(fstaruni(i,t)**2))
+         nrec2 = mm    
+        
       end if
    end do
-   call dgemv('t',m,r,1.0d0,rtv(1:m,1:r,(t-1)*timevar(2)+1),m,rrec,1,0.0d0,r1,1)
-   call dsymv('u',r,1.0d0,qt(1:r,1:r,(t-1)*timevar(3)+1),r,r1,1,0.0d0,etahat(1:r,t),1)
-   etahatvar(1:r,1:r,t) = qt(1:r,1:r,(t-1)*timevar(3)+1)
-   call dsymm('r','u',m,r,1.0d0,qt(1:r,1:r,(t-1)*timevar(3)+1),r,rtv(1:m,1:r,(t-1)*timevar(2)+1),m,0.0d0,mr,m)
-   call dgemm('n','n',m,r,m,1.0d0,nrec,m,mr,m,0.0d0,mr2,m)
-   call dgemm('t','n',r,r,m,1.0d0,mr,m,mr2,m,1.0d0,etahatvar(1:r,1:r,t),r)
+ !  call dgemv('t',m,r,1.0d0,rtv(1:m,1:r,(t-1)*timevar(2)+1),m,rrec,1,0.0d0,r1,1)
+ !  call dsymv('u',r,1.0d0,qt(1:r,1:r,(t-1)*timevar(3)+1),r,r1,1,0.0d0,etahat(1:r,t),1)
+ !  etahatvar(1:r,1:r,t) = qt(1:r,1:r,(t-1)*timevar(3)+1)
+ !  call dsymm('r','u',m,r,1.0d0,qt(1:r,1:r,(t-1)*timevar(3)+1),r,rtv(1:m,1:r,(t-1)*timevar(2)+1),m,0.0d0,mr,m)
+  ! call dgemm('n','n',m,r,m,1.0d0,nrec,m,mr,m,0.0d0,mr2,m)
+  ! call dgemm('t','n',r,r,m,1.0d0,mr,m,mr2,m,1.0d0,etahatvar(1:r,1:r,t),r)
 
    rt0(1:m,t) = rrec
    rt1(1:m,t) = rrec1
@@ -322,6 +319,10 @@ if(d>0) then
        do i = ydimt(t), 1, -1
          
          if(finfuni(i,t)> eps) then
+         !   epshat(i,t) = -ht(i,i,t)/finfuni(i,t)*ddot(m,kinfuni(1:m,i,t),1,rrec,1)
+        !    call dgemv('n',m,m,1.0d0,nrec,m,kinfuni(1:m,i,t),1,0.0d0,rhelp,1)
+        !    epshatvar(i,t) = ht(i,i,t) - (ht(i,i,t)**2)/(finfuni(i,t)**2)*&
+         !        (ftuni(i,t) + ddot(m,kinfuni(1:m,i,t),1,rhelp,1))
             linfuni = im            
             call dger(m,m,-1.0d0/finfuni(i,t),kinfuni(1:m,i,t),1,zt(i,1:m,t),1,linfuni,m) !linf
             rhelp = -1.0d0*kstaruni(1:m,i,t)
@@ -356,11 +357,13 @@ if(d>0) then
             
             call dgemm('t','n',m,m,m,1.0d0,linfuni,m,mm,m,0.0d0,nrec,m) !nt0 = linf'*mm
             
-            epshat(i,t) = -ht(i,i,t)/finfuni(i,t)*ddot(m,kinfuni(1:m,i,t),1,rrec,1)
-            call dgemv('n',m,m,1.0d0,nrec,m,kinfuni(1:m,i,t),1,1.0d0,rhelp,1)
-            epshatvar(i,t) = ht(i,i,t) - (ht(i,i,t)**2)/(finfuni(i,t)**2)*&
-                 (ftuni(i,t) + ddot(m,kinfuni(1:m,i,t),1,rhelp,1))
+           
          else
+         !   epshat(i,t) = ht(i,i,t) * (vtuni(i,t)/fstaruni(i,t) - &
+        !         ddot(m,kinfuni(1:m,i,t),1,rrec,1)/fstaruni(i,t))
+         !   call dgemv('n',m,m,1.0d0,nrec,m,kstaruni(1:m,i,t),1,0.0d0,rhelp,1)
+        !    epshatvar(i,t) = ht(i,i,t) - ht(i,i,t)**2 * (1.0d0/fstaruni(i,t) &
+        !         + ddot(m,kstaruni(1:m,i,t),1,rhelp,1)/(fstaruni(i,t)**2))
             lstaruni= im
             call dger(m,m,(-1.0d0)/fstaruni(i,t),kstaruni(1:m,i,t),1,zt(i,1:m,t),1,lstaruni,m) !lstar = I -Kstar*Z/Fstar         
             call dgemv('t',m,m,1.0d0,lstaruni,m,rrec,1,0.0d0,rhelp,1) !oli beta 1.0d0!!!!... JA miinusmerkki
@@ -375,22 +378,17 @@ if(d>0) then
             call dgemm('n','n',m,m,m,1.0d0,nrec1,m,lstaruni,m,0.0d0,mm,m) !mm = nt1*lstar
             nrec1 = mm
             call dgemm('n','n',m,m,m,1.0d0,nrec2,m,lstaruni,m,0.0d0,mm,m) !mm = nt1*lstar
-            nrec2 = mm
+            nrec2 = mm         
             
-            epshat(i,t) = ht(i,i,t) * (vtuni(i,t)/fstaruni(i,t) - &
-                 ddot(m,kinfuni(1:m,i,t),1,rrec,1)/fstaruni(i,t))
-            call dgemv('n',m,m,1.0d0,nrec,m,kstaruni(1:m,i,t),1,0.0d0,rhelp,1)
-            epshatvar(i,t) = ht(i,i,t) - ht(i,i,t)**2 * (1.0d0/fstaruni(i,t) &
-                 + ddot(m,kstaruni(1:m,i,t),1,rhelp,1)/(fstaruni(i,t)**2))
          end if
       end do
       
-      call dgemv('t',m,r,1.0d0,rtv(1:m,1:r,(t-1)*timevar(2)+1),m,rrec,1,0.0d0,r1,1)
-      call dsymv('u',r,1.0d0,qt(1:r,1:r,(t-1)*timevar(3)+1),r,r1,1,0.0d0,etahat(1:r,t),1)
-      etahatvar(1:r,1:r,t) = qt(1:r,1:r,(t-1)*timevar(3)+1)
-      call dsymm('r','u',m,r,1.0d0,qt(1:r,1:r,(t-1)*timevar(3)+1),r,rtv(1:m,1:r,(t-1)*timevar(2)+1),m,0.0d0,mr,m)
-      call dgemm('n','n',m,r,m,1.0d0,nrec,m,mr,m,0.0d0,mr2,m)
-      call dgemm('t','n',r,r,m,1.0d0,mr,m,mr2,m,1.0d0,etahatvar(1:r,1:r,t),r)
+      !call dgemv('t',m,r,1.0d0,rtv(1:m,1:r,(t-1)*timevar(2)+1),m,rrec,1,0.0d0,r1,1)
+      !call dsymv('u',r,1.0d0,qt(1:r,1:r,(t-1)*timevar(3)+1),r,r1,1,0.0d0,etahat(1:r,t),1)
+      !etahatvar(1:r,1:r,t) = qt(1:r,1:r,(t-1)*timevar(3)+1)
+      !call dsymm('r','u',m,r,1.0d0,qt(1:r,1:r,(t-1)*timevar(3)+1),r,rtv(1:m,1:r,(t-1)*timevar(2)+1),m,0.0d0,mr,m)
+      !call dgemm('n','n',m,r,m,1.0d0,nrec,m,mr,m,0.0d0,mr2,m)
+      !call dgemm('t','n',r,r,m,1.0d0,mr,m,mr2,m,1.0d0,etahatvar(1:r,1:r,t),r)
       
       
       rt0(1:m,t) = rrec
